@@ -9,7 +9,7 @@ use List::MoreUtils qw( any );
 
 use Term::Choose           qw();
 use Term::Choose::LineFold qw( line_fold print_columns );
-use Term::Choose::Util     qw( insert_sep get_term_width unicode_sprintf );
+use Term::Choose::Util     qw( insert_sep get_term_width get_term_height unicode_sprintf );
 use Term::Choose::Screen   qw( clear_screen );
 use Term::Form             qw();
 
@@ -34,41 +34,45 @@ sub input_filter {
     my $back          = '     <<';
     my $reset         = '    RESET';
     my $reparse       = '   REPARSE';
-    my $choose_cols   = 'Choose_Cols';
-    my $choose_rows   = 'Choose_Rows';
+    my $merge_rows    = 'Merge_Rows';
     my $range_rows    = 'Range_Rows';
     my $row_groups    = 'Row_Groups';
+    my $choose_rows   = 'Choose_Rows';
     my $remove_cell   = 'Remove_Cell';
     my $insert_cell   = 'Insert_Cell';
-    my $append_col    = 'Append_Col';
-    my $split_column  = 'Split_Column';
-    my $s_and_replace = 'S_&_Replace';
     my $split_table   = 'Split_Table';
-    my $merge_rows    = 'Merge_Rows';
+    my $split_column  = 'Split_Column';
     my $join_columns  = 'Join_Columns';
     my $fill_up_rows  = 'Fill_up_Rows';
-    my $cols_to_rows  = 'Cols_to_Rows';
-    my $empty_to_null = 'Empty_2_NULL';
+    my $empty_to_null = ' Empty_2_NULL';
+    my $choose_cols   = 'Choose_Columns';
+    my $append_col    = 'Append_Columns';
+    my $cols_to_rows  = 'Columns_to_Rows';
+    my $s_and_replace = 'Search_&_Replace';
     $sf->{empty_to_null} = $sf->{o}{insert}{'empty_to_null_' . $sf->{i}{gc}{source_type}};
-
     my $old_idx = 0;
 
     FILTER: while ( 1 ) {
-        my $count_static_rows = 5; # prompt and 4 menu rows (fix width)
-        $sf->__print_filter_info( $sql, $count_static_rows, undef );
+        my $skip = ' ';
+        my $regex = qr/^\Q$skip\E\z/;
         my $menu = [
-            undef,    $choose_cols,   $choose_rows,  $range_rows, $row_groups,
-            $confirm, $remove_cell,   $insert_cell,  $append_col, $split_column,
-            $reset,   $s_and_replace, $split_table,  $merge_rows, $join_columns,
-            $reparse, $empty_to_null, $fill_up_rows, $cols_to_rows,
+            undef,          $choose_cols,   $skip,         $skip,
+            $confirm,       $choose_rows,   $range_rows,   $row_groups,
+            $reset,         $s_and_replace, $skip,         $skip,
+            $reparse,       $remove_cell,   $insert_cell,  $skip,
+            $empty_to_null, $join_columns,  $split_column, $append_col,
+            $cols_to_rows,  $split_table,   $merge_rows,   $fill_up_rows,
         ];
+        my $max_cols = 4;
+        my $count_static_rows = 2; # prompt + trailing empty line
+        my $info = $sf->__get_filter_info( $sql, $count_static_rows, undef, $menu, $max_cols );
         # Choose
         my $idx = $tc->choose(
             $menu,
-            { prompt => 'Filter:', layout => 0, order => 0, max_width => 78, index => 1, default => $old_idx,
-              undef => $back }
+            { info => $info, prompt => 'Filter:', layout => 0, order => 0, max_cols => $max_cols, index => 1,
+              default => $old_idx, undef => $back, skip_items => $regex, busy_string => $sf->{i}{working},
+              keep => $sf->{i}{keep} }
         );
-        #$sf->__print_filter_info( $sql, $count_static_rows, undef );
         if ( ! $idx ) {
             $sql->{insert_into_args} = []; #
             return;
@@ -83,7 +87,7 @@ sub input_filter {
         my $filter = $menu->[$idx];
         my $filter_str = sprintf( "Filter: %s", $filter );
         if ( $filter eq $reset ) {
-            $sf->__print_filter_info( $sql, $count_static_rows, undef ); #
+            $sf->__print_filter_info( $sql, $count_static_rows, undef, $menu, $max_cols ); #
             $sql->{insert_into_args} = [ map { [ @$_ ] } @{$sf->{i}{gc}{bu_insert_into_args}} ];
             $sf->{empty_to_null} = $sf->{o}{insert}{'empty_to_null_' . $sf->{i}{gc}{source_type}};
             delete $sf->{i}{prev_chosen_cols};
@@ -91,15 +95,13 @@ sub input_filter {
         }
         elsif ( $filter eq $confirm ) {
             if ( $sf->{empty_to_null} ) {
-                $sf->__print_filter_info( $sql, $count_static_rows, undef );
+                $sf->__print_filter_info( $sql, $count_static_rows, undef, $menu, $max_cols );
                 no warnings 'uninitialized';
                 $sql->{insert_into_args} = [ map { [ map { length ? $_ : undef } @$_ ] } @{$sql->{insert_into_args}} ];
             }
             return 1;
         }
         elsif ( $filter eq $reparse ) {
-            my $count_static_rows = 9; # bigges settings sub-menu 9 rows
-            $sf->__print_filter_info( $sql, $count_static_rows, undef );
             return -1;
         }
         elsif ( $filter eq $choose_cols  ) {
@@ -155,29 +157,40 @@ sub input_filter {
 
 
 sub __print_filter_info {
-    my ( $sf, $sql, $count_static_rows, $horizontal_choices ) = @_;
+    my ( $sf, $sql, $count_static_rows, $pre, $choices, $max_cols ) = @_;
+    my $info = $sf->__get_filter_info( $sql, $count_static_rows, $pre, $choices, $max_cols );
+    print clear_screen();
+    say $info;
+    say "";
+    print $sf->{i}{working} . "\r";
+}
+
+
+sub __get_filter_info {
+    my ( $sf, $sql, $count_static_rows, $pre, $choices, $max_cols ) = @_;
     # $count_static_rows not realy static count - some could be line-folded if too long
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     print $sf->{i}{working} . "\r";
     $sf->{i}{occupied_term_height}  = 1; # "DATA:" prompt
-    # ... insert_into_args rows
+    # ...                                # insert_into_args rows
     $sf->{i}{occupied_term_height} += 1; # empty row
     $sf->{i}{occupied_term_height} += $count_static_rows;
-    if ( defined $horizontal_choices ) {
-        my $term_w = get_term_width();
-        my $longest = 0;
-        my @tmp_cols = map{ ! length $_ ? '--' : $_ } @$horizontal_choices;
-        for my $col ( @tmp_cols ) {
-            my $col_w = print_columns( $col );
-            $longest = $col_w if $col_w > $longest;
-        }
-        if ( $longest * 2 + 2 > $term_w ) {
-            $sf->{i}{occupied_term_height} += @tmp_cols;
-        }
-        else {
-            my $r = print_columns( join( ' ' x 2, @tmp_cols ) ) / $term_w;
+    my $row_count_menu = 0;
+    if ( @{$choices//[]} ) {
+        if ( ! $max_cols ) {
+            my $term_w = get_term_width();
+            my $longest = 0;
+            my @tmp_cols = map{ ! length $_ ? '--' : $_ } @{$pre//[]}, @$choices;
+            for my $col ( @tmp_cols ) {
+                my $col_w = print_columns( $col );
+                $longest = $col_w if $col_w > $longest;
+            }
+            my $r = print_columns( join( ' ' x 2, @tmp_cols ) ) / $term_w; ## pad 2
             if ( $r <= 1 ) {
-                $sf->{i}{occupied_term_height} += 1;
+                $row_count_menu = 1;
+            }
+            elsif ( $longest * 2 + 2 > $term_w ) { ## pad 2
+                $row_count_menu = @tmp_cols;
             }
             else {
                 my $joined_cols = $longest;
@@ -186,23 +199,33 @@ sub __print_filter_info {
                     $joined_cols += 2 + $longest;
                     ++$cols_in_a_row;
                 }
-                my $row_count = int( @tmp_cols / $cols_in_a_row );
-                $row_count++ if @tmp_cols % $cols_in_a_row;
-                $sf->{i}{occupied_term_height} += $row_count;
+                $row_count_menu = int( @tmp_cols / $cols_in_a_row );
+                if ( @tmp_cols % $cols_in_a_row ) {
+                    $row_count_menu++;
+                }
+            }
+        }
+        elsif ( $max_cols == 1 ) {
+            $row_count_menu = @{$pre//[]} + @$choices;
+        }
+        else {
+            my $count_items = @{$pre//[]} + @$choices;
+            $row_count_menu = int( $count_items / $max_cols );
+            if ( $count_items % $max_cols ) {
+                $row_count_menu += 1;
             }
         }
     }
-    #$sf->{i}{occupied_term_height} += 1; # to keep bottom line empty
+    $sf->{i}{occupied_term_height} += $row_count_menu;
+#    if ( $sf->{i}{occupied_term_height} + 1 < get_term_height()  ) {
+        $sf->{i}{keep} = 1; ##;
+#    }
     my $indent = '';
     my $bu_stmt_types = [ @{$sf->{i}{stmt_types}} ];
     $sf->{i}{stmt_types} = [];
     my $rows = $ax->insert_into_args_info_format( $sql, $indent );
     $sf->{i}{stmt_types} = $bu_stmt_types;
-    print clear_screen();
-    say "DATA:";
-    say $_ for @$rows;
-    say "";
-    print $sf->{i}{working} . "\r";
+    return join( "\n", 'DATA:', @$rows ) . "\n"; # "\n" == empty line after insert_args
 }
 
 
@@ -212,8 +235,8 @@ sub __choose_columns {
     my $aoa = $sql->{insert_into_args};
     my $empty_cells_of_col_count = $sf->__count_empty_cells_of_cols( $aoa );
     my $header = $sf->__prepare_header( $aoa, $empty_cells_of_col_count );
-    my $count_static_rows = 2; # filter_str and cs_label
-    $sf->__print_filter_info( $sql, $count_static_rows, [ '<<', $sf->{i}{ok}, @$header ] );
+    my $count_static_rows = 3; # filter_str + cs_label + trailing empty line
+    $sf->__print_filter_info( $sql, $count_static_rows, [ '<<', $sf->{i}{ok} ], $header, undef );
     my $row_count = @$aoa;
     my $col_count = @{$aoa->[0]};
     my $mark = [];
@@ -233,10 +256,14 @@ sub __choose_columns {
         }
         $mark = $mark2 if @$mark2 == @$prev_chosen;
     }
+    my $back = '<<';
+    my $info = $sf->__get_filter_info( $sql, $count_static_rows, [ $back, $sf->{i}{ok} ], $header, undef ) . "\n" . $filter_str;
+    # Choose
     my $col_idx = $tu->choose_a_subset(
         $header,
         { cs_label => 'Cols: ', layout => 0, order => 0, mark => $mark, all_by_default => 1, index => 1,
-            confirm => $sf->{i}{ok}, back => '<<', info => $filter_str, busy_string => $sf->{i}{working} }
+          confirm => $sf->{i}{ok}, back => $back, info => $info, keep => $sf->{i}{keep},
+          busy_string => $sf->{i}{working} }
     );
     if ( ! defined $col_idx ) {
         return;
@@ -252,29 +279,30 @@ sub __choose_rows {
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $aoa = $sql->{insert_into_args};
-    my $count_static_rows = 4 + @$aoa; # filter_str, prompt, back, confirm and aoa
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
-    my @pre = ( undef, $sf->{i}{ok} );
-    my @stringified_rows;
+    my $count_static_rows = 3; # filter_str, prompt, trailing empty line
+    my @pre = ( undef, $sf->{i}{ok} ); # back, confirm
+    $sf->__print_filter_info( $sql, $count_static_rows, [ ( ' ' ) x @pre ], [ ( ' ' ) x @$aoa ], 1 ); ##
+    my $stringified_rows = [];
     my $mark;
     {
         no warnings 'uninitialized';
         for my $i ( 0 .. $#$aoa ) {
             push @$mark, $i + @pre if length join '', @{$aoa->[$i]};
-            push @stringified_rows, join ',', @{$aoa->[$i]};
+            push @$stringified_rows, join ',', @{$aoa->[$i]};
         }
     }
-    if ( @$mark == @stringified_rows ) {
+    if ( @$mark == @$stringified_rows ) {
         $mark = undef;
     }
     my $prompt = 'Choose rows:';
-    $sql->{insert_into_args} = []; # $sql->{insert_into_args} refers to a new new empty array - this doesn't delete $aoa
+    $sql->{insert_into_args} = []; # $sql->{insert_into_args} refers to a new empty array - this doesn't delete $aoa
 
     while ( 1 ) {
+        my $info = $sf->__get_filter_info( $sql, $count_static_rows, [ ( ' ' ) x @pre ], [ ( ' ' ) x @$aoa ], 1 ) . "\n" . $filter_str;
         # Choose
         my @idx = $tc->choose(
-            [ @pre, @stringified_rows ],
-            { %{$sf->{i}{lyt_v}}, prompt => $prompt, info => $filter_str, meta_items => [ 0 .. $#pre ],
+            [ @pre, @$stringified_rows ],
+            { %{$sf->{i}{lyt_v}}, prompt => $prompt, info => $info, meta_items => [ 0 .. $#pre ],
               include_highlighted => 2, index => 1, undef => '<<', busy_string => $sf->{i}{working}, mark => $mark }
         );
         $sf->__print_filter_info( $sql, $count_static_rows, undef );
@@ -303,26 +331,27 @@ sub __choose_rows {
 sub __range_of_rows {
     my ( $sf, $sql, $filter_str ) = @_;
     my $aoa = $sql->{insert_into_args};
-    my $count_static_rows = 3 + @$aoa; # filter_str, prompt, back and aoa
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
-    # Choose
+    my $count_static_rows = 3; # filter_str, prompt, trailing empty line
+    my $pre_count = 1; # back
+    $sql->{insert_into_args} = []; # temporarily: because the rows are the choices
+    my $info = $sf->__get_filter_info( $sql, $count_static_rows,  [ ( ' ' ) x $pre_count ], [ ( ' ' ) x @$aoa ], 1 ) . "\n" . $filter_str;
     my $prompt = "Choose first row:";
     # Choose
-    my $idx_first_row = $sf->__choose_a_row_idx( $aoa, $filter_str, $prompt );
+    my $idx_first_row = $sf->__choose_a_row_idx( $aoa, $info, $prompt );
     if ( ! defined $idx_first_row ) {
+        $sql->{insert_into_args} = $aoa;
         return;
     }
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
+    $sql->{insert_into_args} = [ $aoa->[$idx_first_row] ]; # temporarily for the info output
+    $info = $sf->__get_filter_info( $sql, $count_static_rows,  [ ( ' ' ) x $pre_count ], [ ( ' ' ) x @$aoa ], 1 ) . "\n" . $filter_str;
     $prompt = "Choose last row:";
     # Choose
-    my $idx_last_row = $sf->__choose_a_row_idx( [ @{$aoa}[$idx_first_row .. $#$aoa] ], $filter_str, $prompt );
+    my $idx_last_row = $sf->__choose_a_row_idx( [ @{$aoa}[$idx_first_row .. $#$aoa] ], $info, $prompt );
     if ( ! defined $idx_last_row ) {
+        $sql->{insert_into_args} = $aoa;
         return;
     }
-    if ( ! defined $idx_first_row || ! defined $idx_last_row ) {
-        return;
-    }
-    $sf->__print_filter_info( $sql, $count_static_rows, undef ); #
+    $sf->__print_filter_info( $sql, $count_static_rows,  [ ( ' ' ) x $pre_count ], [ ( ' ' ) x @$aoa ], 1 ); #
     $idx_last_row += $idx_first_row;
     $sql->{insert_into_args} = [ @{$aoa}[$idx_first_row .. $idx_last_row] ];
     return 1;
@@ -350,16 +379,18 @@ sub __row_groups {
             $len, insert_sep( $row_count, $sf->{o}{G}{thsd_sep} ), $row_str,
             $col_count, $col_str;
     }
-    my $count_static_rows = 7 + @choices_groups; # filter_str, prompt, cs_label x 2, cs_end, back, confirm and choices_groups
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
+    my $count_static_rows = 6; # filter_str, prompt, 2 x cs_label, cs_end, trailing empty line
+    my $pre_count = 2; # back and confirm
+    my $info = $sf->__get_filter_info( $sql, $count_static_rows,  [ ( ' ' ) x $pre_count ], [ ( ' ' ) x @choices_groups ], 1 ) . "\n" . $filter_str;
     my $prompt = 'Choose group:';
+    # Choose
     my $idxs = $tu->choose_a_subset(
         \@choices_groups,
-        { info => $filter_str, prompt => $prompt, layout => 3, index => 1, confirm => $sf->{i}{ok},
+        { info => $info, prompt => $prompt, layout => 3, index => 1, confirm => $sf->{i}{ok},
           back => '<<', all_by_default => 1, cs_label => "Chosen groups:\n", cs_separator => "\n",
-          cs_end => "\n", busy_string => $sf->{i}{working} }
+          cs_end => "\n", busy_string => $sf->{i}{working}, keep => $sf->{i}{keep} }
     );
-    $sf->__print_filter_info( $sql, $count_static_rows, undef ); #
+    $sf->__print_filter_info( $sql, $count_static_rows,  [ ( ' ' ) x $pre_count ], [ ( ' ' ) x @choices_groups ], 1 ); #
     if ( ! defined $idxs ) {
         return;
     }
@@ -380,19 +411,18 @@ sub __remove_cell {
     my $aoa = $sql->{insert_into_args};
 
     while ( 1 ) {
-        my $count_static_rows = 3 + @$aoa; # filter_str, prompt, back and aoa
-        $sf->__print_filter_info( $sql, $count_static_rows, undef );
+        my $info = $filter_str;
         my $prompt = "Choose row:";
         # Choose
-        my $row_idx = $sf->__choose_a_row_idx( $aoa, $filter_str, $prompt );
+        my $row_idx = $sf->__choose_a_row_idx( $aoa, $info, $prompt );
         if ( ! defined $row_idx ) {
             return;
         }
+        my $count_static_rows = 3; # filter_str and prompt, trailing empty line
+        $info = $sf->__get_filter_info( $sql, $count_static_rows, [ '<<' ], $aoa->[$row_idx], undef ) . "\n" . $filter_str;
         $prompt = "Choose cell:";
-        $count_static_rows = 2; # filter_str and prompt
-        $sf->__print_filter_info( $sql, $count_static_rows, [ '<<', @{$aoa->[$row_idx]} ] );
         # Choose
-        my $col_idx = $sf->__choose_a_column_idx( [ @{$aoa->[$row_idx]} ], $filter_str, $prompt );
+        my $col_idx = $sf->__choose_a_column_idx( [ @{$aoa->[$row_idx]} ], $info, $prompt );
         if ( ! defined $col_idx ) {
             next;
         }
@@ -417,20 +447,19 @@ sub __insert_cell {
     my $aoa = $sql->{insert_into_args};
 
     while ( 1 ) {
-        my $count_static_rows = 3 + @$aoa; # filter_str, prompt, back and aoa
-        $sf->__print_filter_info( $sql, $count_static_rows, undef );
+        my $info = $filter_str;
         my $prompt = "Choose row:";
         # Choose
-        my $row_idx = $sf->__choose_a_row_idx( $aoa, $filter_str, $prompt );
+        my $row_idx = $sf->__choose_a_row_idx( $aoa, $info, $prompt );
         if ( ! defined $row_idx ) {
             return;
         }
         my $cols = [ @{$aoa->[$row_idx]}, 'END_of_Row' ];
-        $count_static_rows = 2; # prompt and filter_str
-        $sf->__print_filter_info( $sql, $count_static_rows, [ '<<', @$cols ] );
+        my $count_static_rows = 3; # prompt, filter_str, trailing empty line
+        $info = $sf->__get_filter_info( $sql, $count_static_rows, [ '<<' ], $cols, undef ) . "\n" . $filter_str;
         $prompt = "Insert cell before:";
         # Choose
-        my $col_idx = $sf->__choose_a_column_idx( $cols, $filter_str, $prompt );
+        my $col_idx = $sf->__choose_a_column_idx( $cols, $info, $prompt );
         if ( ! defined $col_idx ) {
             next;
         }
@@ -440,16 +469,19 @@ sub __insert_cell {
         $str_row_with_placeholder =~ s/"<\*>"/<*>/;
         my $term_w = get_term_width();
         my $label = 'Row: ';
-        my @info = ( $filter_str );
-        push @info, line_fold(
+        my @tmp_info = ( $filter_str );
+        push @tmp_info, line_fold(
             $label . $str_row_with_placeholder, $term_w,
             { subseq_tab => ' ' x length $label, join => 0 }
         );
+        $count_static_rows = @tmp_info + 2; # tmp_info, readline, trailing empty line
+        $info = $sf->__get_filter_info( $sql, $count_static_rows, undef, undef, 1 ). "\n" . join( "\n", @tmp_info );
         $prompt = "<*>: ";
-        $count_static_rows = @info + 1; # info and readline
-        $sf->__print_filter_info( $sql, $count_static_rows, undef );
         # Readline
-        my $cell = $tf->readline( $prompt, { info => join( "\n", @info ) } );
+        my $cell = $tf->readline(
+            $prompt,
+            { info => $info }
+        );
         splice( @{$aoa->[$row_idx]}, $col_idx, 0, $cell );
         $sql->{insert_into_args} = $aoa;
         return;
@@ -461,14 +493,15 @@ sub __fill_up_rows {
     my ( $sf, $sql, $filter_str ) = @_;
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my $aoa = $sql->{insert_into_args};
-    my $count_static_rows = 4; # filter_str, prompt, back and confirm
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
+    my $count_static_rows = 3; # filter_str, prompt, trailing empty line
+    my $menu = [ undef, '- YES' ]; # back, confirm
+    my $info = $sf->__get_filter_info( $sql, $count_static_rows,  undef, [ ( ' ' ) x @$menu ], 1 ) . "\n" . $filter_str;
     my $prompt = 'Fill up shorter rows?';
     my $ok = $tc->choose(
-        [ undef, '- YES' ],
-        { info => $filter_str, prompt => $prompt, index => 1, undef => '- NO', layout => 3 }
+        $menu,
+        { info => $info, prompt => $prompt, index => 1, undef => '- NO', layout => 3, keep => $sf->{i}{keep} }
     );
-    $sf->__print_filter_info( $sql, $count_static_rows, undef ); #
+    $sf->__print_filter_info( $sql, $count_static_rows, undef, [ ( ' ' ) x @$menu ], 1 ); #
     if ( ! $ok ) {
         return;
     }
@@ -492,14 +525,15 @@ sub __append_col {
     my ( $sf, $sql, $filter_str ) = @_;
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my $aoa = $sql->{insert_into_args};
-    my $count_static_rows = 4; # filter_str, prompt, back and confirm
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
+    my $count_static_rows = 3; # filter_str, prompt, trailing empty line
+    my $menu = [ undef, '- YES' ]; # back, confirm
+    my $info = $sf->__get_filter_info( $sql, $count_static_rows, undef, [ ( ' ' ) x @$menu ], 1 ) . "\n" . $filter_str;
     my $prompt = 'Append an empty column?';
     my $ok = $tc->choose(
-        [ undef, '- YES' ],
-        { info => $filter_str, prompt => $prompt, index => 1, undef => '- NO', layout => 3 }
+        $menu,
+        { info => $info, prompt => $prompt, index => 1, undef => '- NO', layout => 3, keep => $sf->{i}{keep} }
     );
-    $sf->__print_filter_info( $sql, $count_static_rows, undef ); #
+    $sf->__print_filter_info( $sql, $count_static_rows,  undef, [ ( ' ' ) x @$menu ], 1 ); #
     if ( $ok ) {
         my $new_last_idx = $#{$aoa->[0]} + 1;
         for my $row ( @$aoa ) {
@@ -517,30 +551,30 @@ sub __split_column {
     my $aoa = $sql->{insert_into_args};
     my $empty_cells_of_col_count =  $sf->__count_empty_cells_of_cols( $aoa );
     my $header = $sf->__prepare_header( $aoa, $empty_cells_of_col_count );
-    my $count_static_rows = 2; # filter_str and prompt
-    $sf->__print_filter_info( $sql, $count_static_rows, [ '<<', @$header ] );
+    my $count_static_rows = 3; # filter_str, prompt, trailing empty line
+    my $info = $sf->__get_filter_info( $sql, $count_static_rows, [ '<<' ], $header, undef ) . "\n" . $filter_str;
     my $prompt = 'Choose column:';
     # Choose
-    my $idx = $sf->__choose_a_column_idx( $header, $filter_str, $prompt );
+    my $idx = $sf->__choose_a_column_idx( $header, $info, $prompt );
     if ( ! defined $idx ) {
         return;
     }
-    my $info = $filter_str;
-    $prompt = "Split column \"$header->[$idx]\"";
     my $fields = [
         [ 'Pattern', ],
         [ 'Limit', ],
         [ 'Left trim', '\s+' ],
         [ 'Right trim', '\s+' ]
     ];
+    $count_static_rows = 3; # filter_str, prompt, trailing empty line
+    my $pre_count = 2; # back, confirm
+    $info = $sf->__get_filter_info( $sql, $count_static_rows, [ ( ' ' ) x $pre_count ], [ ( ' ' ) x @$fields ], 1 ) . "\n" . $filter_str;
+    $prompt = "Split column \"$header->[$idx]\"";
     my $c;
-    $count_static_rows = 4 + @$fields; # filter_str, prompt, back, confirm and fields
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
     # Fill_form
     my $form = $tf->fill_form(
         $fields,
-        { info => $info, prompt => $prompt,
-        auto_up => 2, confirm => $sf->{i}{confirm}, back => $sf->{i}{back} . '   ' }
+        { info => $info, prompt => $prompt, auto_up => 2, confirm => $sf->{i}{confirm}, back => $sf->{i}{back} . '   ',
+          keep => $sf->{i}{keep} }
     );
     if ( ! $form ) {
         return;
@@ -574,28 +608,31 @@ sub __split_table {
     my $tu = Term::Choose::Util->new( $sf->{i}{tcu_default} );
     my $aoa = $sql->{insert_into_args};
     my $digits = length( scalar @{$aoa->[0]} );
-    my $count_static_rows = 4 + $digits; # filter_str, prompt, back, confirm and digits
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
+    my $count_static_rows = 3; # filter_str, prompt, trailing empty line
+    my $pre_count = 2; # back confirm
+    my $info = $sf->__get_filter_info( $sql, $count_static_rows, [ ( ' ' ) x $pre_count ], [ ( ' ' ) x $digits ], 1 ) . "\n" . $filter_str;
     # Choose
     my $col_count = $tu->choose_a_number(
         $digits,
-        { info => $filter_str, cs_label => 'Number columns new table: ', small_first => 1 }
+        { info => $info, cs_label => 'Number columns new table: ', small_first => 1, keep => $sf->{i}{keep} }
     );
     if ( ! defined $col_count ) {
         return;
     }
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
+    $sf->__print_filter_info( $sql, $count_static_rows, [ ( ' ' ) x $pre_count ], [ ( ' ' ) x $digits ], 1 );
     if ( @{$aoa->[0]} < $col_count ) {
+        my $info = $sf->__get_filter_info( $sql, $count_static_rows, [ ( ' ' ) x $pre_count ], [ ( ' ' ) x $digits ], 1 ) . "\n" . $filter_str; ##
         $tc->choose(
             [ 'Chosen number bigger than the available columns!' ],
-            { prompt => 'Close with ENTER' }
+            { info => $info, prompt => 'Close with ENTER', keep => $sf->{i}{keep} }
         );
         return;
     }
     if ( @{$aoa->[0]} % $col_count ) {
+        my $info = $sf->__get_filter_info( $sql, $count_static_rows, [ ( ' ' ) x $pre_count ], [ ( ' ' ) x $digits ], 1 ) . "\n" . $filter_str; ##
         $tc->choose(
             [ 'The number of available columns cannot be divided by the chosen number without rest!' ],
-            { prompt => 'Close with ENTER' }
+            { info => $info, prompt => 'Close with ENTER', keep => $sf->{i}{keep} }
         );
         return;
     }
@@ -622,8 +659,9 @@ sub __merge_rows {
     my $tu = Term::Choose::Util->new( $sf->{i}{tcu_default} );
     my $tf = Term::Form->new( $sf->{i}{tf_default} );
     my $aoa = $sql->{insert_into_args};
-    my $count_static_rows = 4 + @$aoa; # filter_str, prompt, back, confirm and aoa
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
+
+    my $info = $filter_str;
+    print "\r$info"; ##
     my $term_w = get_term_width();
     my @stringified_rows;
     {
@@ -639,16 +677,19 @@ sub __merge_rows {
         } @$aoa;
     }
     my $prompt = 'Choose rows:';
+    # Choose
     my $chosen_idxs = $tu->choose_a_subset(
         \@stringified_rows,
         { cs_separator => "\n", cs_end => "\n", layout => 3, order => 0, all_by_default => 0, prompt => $prompt,
-          index => 1, confirm => $sf->{i}{ok}, back => '<<', info => $filter_str, busy_string => $sf->{i}{working} }
+          index => 1, confirm => $sf->{i}{ok}, back => '<<', info => $info, keep => $sf->{i}{keep},
+          busy_string => $sf->{i}{working} }
     );
     if ( ! defined $chosen_idxs || ! @$chosen_idxs ) {
         return;
     }
-    $count_static_rows = 4 + @{$aoa->[$chosen_idxs->[0]]}; # filter_str, prompt, back, confirm and chosen_subset
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
+    my $count_static_rows = 3; # filter_str, prompt, trailing empty line
+    my $pre_count = 2; # back, confirm
+    $sf->__print_filter_info( $sql, $count_static_rows, [ ( ' ' ) x $pre_count ], [ ( ' ' ) x @{$aoa->[$chosen_idxs->[0]]} ], 1 );
     my $merged = [];
     for my $col ( 0 .. $#{$aoa->[$chosen_idxs->[0]]} ) {
         my @tmp;
@@ -662,16 +703,17 @@ sub __merge_rows {
     }
     my $col_number = 0;
     my $fields = [ map { [ ++$col_number, defined $_ ? "$_" : '' ] } @$merged ];
+    $info = $sf->__get_filter_info( $sql, $count_static_rows, [ ( ' ' ) x $pre_count ], [ ( ' ' ) x @{$fields} ], 1 ) . "\n" . $filter_str;
     # Fill_form
     my $form = $tf->fill_form(
         $fields,
-        { info => $filter_str, prompt => 'Edit cells of merged rows:',
+        { info => $info, prompt => 'Edit cells of merged rows:', keep => $sf->{i}{keep},
           auto_up => 2, confirm => $sf->{i}{_confirm}, back => $sf->{i}{_back} . '   ' }
     );
     if ( ! $form ) {
         return;
     }
-    $sf->__print_filter_info( $sql, $count_static_rows, undef ); #
+    $sf->__print_filter_info( $sql, $count_static_rows, [ ( ' ' ) x $pre_count ], [ ( ' ' ) x @{$fields} ], 1 ); #
     $merged = [ map { $_->[1] } @$form ];
     my $first_idx = shift @$chosen_idxs;
     $aoa->[$first_idx] = $merged; # modifies $aoa
@@ -690,32 +732,36 @@ sub __join_columns {
     my $aoa = $sql->{insert_into_args};
     my $empty_cells_of_col_count =  $sf->__count_empty_cells_of_cols( $aoa );
     my $header = $sf->__prepare_header( $aoa, $empty_cells_of_col_count );
-    my $count_static_rows = 2; # filter_str and cs_label
-    $sf->__print_filter_info( $sql, $count_static_rows, [ '<<', $sf->{i}{ok}, @$header ] );
+    my $count_static_rows = 3; # filter_str, cs_label, trailing empty line
+    my $info = $sf->__get_filter_info( $sql, $count_static_rows, [ '<<', $sf->{i}{ok} ], $header, undef ) . "\n" . $filter_str;
     # Choose
     my $chosen_idxs = $tu->choose_a_subset(
         $header,
-        { cs_label => 'Cols: ', layout => 0, order => 0, index => 1, confirm => $sf->{i}{ok},
-          back => '<<', info => $filter_str, busy_string => $sf->{i}{working} }
+        { cs_label => 'Cols: ', layout => 0, order => 0, index => 1, confirm => $sf->{i}{ok}, keep => $sf->{i}{keep},
+          back => '<<', info => $info, busy_string => $sf->{i}{working} }
     );
     if ( ! defined $chosen_idxs || ! @$chosen_idxs ) {
         return;
     }
-    my @info = ( $filter_str );
+    my @tmp_info = ( $filter_str );
     my $label = 'Cols: ';
-    push @info, line_fold(
+    push @tmp_info, line_fold(
         $label . '"' . join( '", "', @{$header}[@$chosen_idxs] ) . '"', get_term_width(),
         { subseq_tab => ' ' x length $label, join => 0 }
     );
-    $count_static_rows = @info + 1; # info_rows and readline
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
+    $count_static_rows = @tmp_info + 2; # tmp_info, readline, empty line
+    $info = $sf->__get_filter_info( $sql, $count_static_rows, undef, undef, 1 ). "\n" . join( "\n", @tmp_info );
     # Readline
-    my $join_char = $tf->readline( 'Join-string: ', { info => join( "\n", @info ) } );
+    my $join_char = $tf->readline(
+        'Join-string: ',
+        { info => $info }
+    );
     if ( ! defined $join_char ) {
         return;
     }
-    $count_static_rows = 4 + @$aoa; # filter_str, prompt, back, confirm and aoa
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
+    $count_static_rows = 3; # filter_str, prompt, trailing empty line
+    my $pre_count = 2; # back, confirm
+    $sf->__print_filter_info( $sql, $count_static_rows, [ ( ' ' ) x $pre_count ], [ ( ' ' ) x @{$aoa} ], 1 );
     my $merged = [];
     for my $row ( 0 .. $#{$aoa} ) {
         my @tmp;
@@ -729,16 +775,17 @@ sub __join_columns {
     }
     my $col_number = 0;
     my $fields = [ map { [ ++$col_number, defined $_ ? "$_" : '' ] } @$merged ];
+    $info = $sf->__get_filter_info( $sql, $count_static_rows, [ ( ' ' ) x $pre_count ], [ ( ' ' ) x @{$fields} ], 1 ) . "\n" . $filter_str;
     # Fill_form
     my $form = $tf->fill_form(
         $fields,
-        { info => $filter_str, prompt => 'Edit cells of joined cols:', auto_up => 2,
+        { info => $info, prompt => 'Edit cells of joined cols:', auto_up => 2, keep => $sf->{i}{keep},
           confirm => $sf->{i}{_confirm}, back => $sf->{i}{_back} . '   ' }
     );
     if ( ! $form ) {
         return;
     }
-    $sf->__print_filter_info( $sql, $count_static_rows, undef ); #
+    $sf->__print_filter_info( $sql, $count_static_rows, [ ( ' ' ) x $pre_count ], [ ( ' ' ) x @{$fields} ], 1 ); #
     $merged = [ map { $_->[1] } @$form ];
     my $first_idx = shift @$chosen_idxs;
     for my $row ( 0 .. $#{$aoa} ) { # modifies $aoa
@@ -756,14 +803,16 @@ sub __transpose_rows_to_cols {
     my ( $sf, $sql, $filter_str ) = @_;
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my $aoa = $sql->{insert_into_args};
-    my $count_static_rows = 4; # filter_str, prompt, back and confirm
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
+    my $menu = [ undef, '- YES' ];
+    my $count_static_rows = 3; # filter_str, prompt, trailing empty line
+    my $info = $sf->__get_filter_info( $sql, $count_static_rows, undef, [ ( ' ' ) x @$menu ], 1 ) . "\n" . $filter_str;
     my $prompt = 'Transpose columns to rows?';
     my $ok = $tc->choose(
-        [ undef, '- YES' ],
-        { info => $filter_str, prompt => $prompt, index => 1, undef => '- NO', layout => 3, busy_string => $sf->{i}{working} }
+        $menu,
+        { info => $info, prompt => $prompt, index => 1, undef => '- NO', layout => 3, busy_string => $sf->{i}{working},
+          keep => $sf->{i}{keep} }
     );
-    $sf->__print_filter_info( $sql, $count_static_rows, undef ); #
+    $sf->__print_filter_info( $sql, $count_static_rows, undef, [ ( ' ' ) x @$menu ], 1 ); #
     if ( $ok ) {
         my $tmp_aoa = [];
         for my $row ( 0 .. $#$aoa ) {
@@ -782,13 +831,14 @@ sub __empty_to_null {
     my $menu_elements = [
         [ 'empty_to_null', "  Empty fields to NULL", [ 'NO', 'YES' ] ]
     ];
-    my $count_static_rows = 3 + @$menu_elements; # prompt, back, confirm and menu_elements
-    $sf->__print_filter_info( $sql, $count_static_rows, undef );
+    my $count_static_rows = 2; # prompt, trailing empty line
+    my $pre_count = 2; # back, confirm
+    my $info = $sf->__get_filter_info( $sql, $count_static_rows, [ ( ' ' ) x $pre_count ], [ ( ' ' ) x @$menu_elements ], 1 );
     my $tmp = { empty_to_null => $sf->{empty_to_null} };
     $tu->settings_menu(
         $menu_elements,
         $tmp,
-        { back => $sf->{i}{_back}, confirm => $sf->{i}{_confirm} }
+        { info => $info, back => $sf->{i}{_back}, confirm => $sf->{i}{_confirm}, keep => $sf->{i}{keep} }
     );
     $sf->{empty_to_null} = $tmp->{empty_to_null};
 }
@@ -833,13 +883,14 @@ sub __prepare_header {
 }
 
 sub __choose_a_column_idx {
-    my ( $sf, $columns, $filter_str, $prompt ) = @_;
+    my ( $sf, $columns, $info, $prompt ) = @_;
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my @pre = ( undef );
     # Choose
     my $col_idx = $tc->choose(
         [ @pre, map( defined $_ ? $_ : '', @$columns ) ],
-        { layout => 0, order => 0, index => 1, undef => '<<', info => $filter_str, prompt => $prompt, empty => '--', busy_string => $sf->{i}{working} } #
+        { layout => 0, order => 0, index => 1, undef => '<<', info => $info, prompt => $prompt, empty => '--',
+          keep => $sf->{i}{keep}, busy_string => $sf->{i}{working} } #
     );
     if ( ! $col_idx ) {
         return;
@@ -848,7 +899,7 @@ sub __choose_a_column_idx {
 }
 
 sub __choose_a_row_idx {
-    my ( $sf, $aoa, $filter_str, $prompt ) = @_;
+    my ( $sf, $aoa, $info, $prompt ) = @_;
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my @stringified_rows;
     {
@@ -859,7 +910,8 @@ sub __choose_a_row_idx {
     # Choose
     my $row_idx = $tc->choose(
         [ @pre, @stringified_rows ],
-        { layout => 3, index => 1, undef => '<<', info => $filter_str, prompt => $prompt, busy_string => $sf->{i}{working} }
+        { layout => 3, index => 1, undef => '<<', info => $info, prompt => $prompt, ## keep => $sf->{i}{keep},
+          busy_string => $sf->{i}{working} }
     );
     if ( ! $row_idx ) {
         return;
