@@ -67,7 +67,6 @@ sub _options {
         group_behavior => [
             { name => '_menu_memory',  text => "- Menu memory",       section => 'G'     },
             { name => '_table_expand', text => "- Expand table",      section => 'table' },
-            { name => '_info_expand',  text => "- Expand info-table", section => 'G'     },
             { name => '_f3',           text => "- F3 filter",         section => 'table' },
             { name => '_mouse',        text => "- Mouse mode",        section => 'table' },
         ],
@@ -102,6 +101,7 @@ sub _options {
             { name => '_file_find_warnings', text => "- Warnings",        section => 'G'     },
         ],
         group_insert => [
+            { name => '_data_source_type',  text => "- Source type of input data",     section => 'insert' },
             { name => '_parse_file',        text => "- Parse tool for 'file'",         section => 'insert' },
             { name => '_parse_copy',        text => "- Parse tool for 'copy & paste'", section => 'insert' },
             { name => '_csv_char',          text => "- csv settings-a",                section => 'csv'    },
@@ -110,7 +110,6 @@ sub _options {
             { name => '_input_filter',      text => "- Enable input filter",           section => 'insert' },
             { name => '_empty_to_null',     text => "- Empty to NULL",                 section => 'insert' },
             { name => '_file_encoding',     text => "- File encoding",                 section => 'insert' },
-            { name => '_data_source_type',  text => "- Source type of input data",     section => 'insert' },
             { name => 'history_dirs',       text => "- Directory history",             section => 'insert' },
             { name => '_file_filter',       text => "- File filter",                   section => 'insert' },
             { name => '_show_hidden_files', text => "- Show hidden files",             section => 'insert' },
@@ -466,15 +465,6 @@ sub set_options {
                 ];
                 $sf->__settings_menu_wrap( $section, $sub_menu, $prompt );
             }
-            elsif ( $opt eq '_info_expand' ) {
-                my $prompt = "Info-tables show the affected rows in 'DROP table', 'DROP view', ";
-                $prompt .= "\n'UPDATE table ...' and 'DELETE FROM table ...'";
-                $prompt .= "\n\n" . 'Choose: ';
-                my $sub_menu = [
-                    [ 'info_expand', "- Expand info-table rows",   [ $no, $yes . ' - fast back', $yes ] ],
-                ];
-                $sf->__settings_menu_wrap( $section, $sub_menu, $prompt );
-            }
             elsif ( $opt eq '_f3' ) {
                 my $prompt = 'Choose: ';
                 my $sub_menu = [
