@@ -57,12 +57,12 @@ sub select {
     my @bu;
 
     COLUMNS: while ( 1 ) {
-        $ax->print_sql( $sql );
+        my $info = $ax->get_sql_info( $sql );
         # Choose
         my @idx = $tc->choose(
             $menu,
-            { %{$sf->{i}{lyt_h}}, meta_items => [ 0 .. $#pre - 1 ], no_spacebar => [ $#pre ], include_highlighted => 2,
-              index => 1 }
+            { %{$sf->{i}{lyt_h}}, info => $info, meta_items => [ 0 .. $#pre - 1 ], no_spacebar => [ $#pre ],
+              include_highlighted => 2, index => 1 }
         );
         if ( ! $idx[0] ) {
             if ( @bu ) {
@@ -102,11 +102,11 @@ sub distinct {
 
     DISTINCT: while ( 1 ) {
         my $menu = [ @pre, $sf->{distinct}, $sf->{all} ];
-        $ax->print_sql( $sql );
+        my $info = $ax->get_sql_info( $sql );
         # Choose
         my $select_distinct = $tc->choose(
             $menu,
-            { %{$sf->{i}{lyt_h}} }
+            { %{$sf->{i}{lyt_h}}, info => $info }
         );
         if ( ! defined $select_distinct ) {
             if ( $sql->{distinct_stmt} ) {
@@ -150,11 +150,11 @@ sub __add_aggregate_substmt {
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my @pre = ( undef, $sf->{i}{ok} );
     my $i = @{$sql->{aggr_cols}};
-    $ax->print_sql( $sql );
+    my $info = $ax->get_sql_info( $sql );
     # Choose
     my $aggr = $tc->choose(
         [ @pre, @{$sf->{aggregate}} ],
-        { %{$sf->{i}{lyt_h}} }
+        { %{$sf->{i}{lyt_h}}, info => $info }
     );
     if ( ! defined $aggr ) {
         return;
@@ -169,11 +169,11 @@ sub __add_aggregate_substmt {
         $aggr =~ s/\(\S\)\z//; #
         $sql->{aggr_cols}[$i] = $aggr . "(";
         if ( $aggr =~ /^(?:COUNT|GROUP_CONCAT|STRING_AGG)\z/ ) {
-            $ax->print_sql( $sql );
+            my $info = $ax->get_sql_info( $sql );
             # Choose
             my $all_or_distinct = $tc->choose(
                 [ undef, $sf->{all}, $sf->{distinct} ],
-                { %{$sf->{i}{lyt_h}} }
+                { %{$sf->{i}{lyt_h}}, info => $info }
             );
             if ( ! defined $all_or_distinct ) {
                 return;
@@ -182,11 +182,11 @@ sub __add_aggregate_substmt {
                 $sql->{aggr_cols}[$i] .= $sf->{distinct};
             }
         }
-        $ax->print_sql( $sql );
+        my $info = $ax->get_sql_info( $sql );
         # Choose
         my $f_col = $tc->choose(
             [ undef, @{$sql->{cols}} ],
-            { %{$sf->{i}{lyt_h}} }
+            { %{$sf->{i}{lyt_h}}, info => $info }
         );
         if ( ! defined $f_col ) {
             return;
@@ -220,11 +220,11 @@ sub set {
     my @pre = ( undef, $sf->{i}{ok} );
 
     COL: while ( 1 ) {
-        $ax->print_sql( $sql );
+        my $info = $ax->get_sql_info( $sql );
         # Choose
         my $quote_col = $tc->choose(
             [ @pre, @{$sql->{cols}} ],
-            { %{$sf->{i}{lyt_h}} }
+            { %{$sf->{i}{lyt_h}}, info => $info }
         );
         if ( ! defined $quote_col ) {
             if ( @bu_col ) {
@@ -287,11 +287,11 @@ sub where {
         if ( $sf->{o}{enable}{parentheses} ) {
             unshift @choices, $unclosed ? ')' : '(';
          }
-        $ax->print_sql( $sql );
+        my $info = $ax->get_sql_info( $sql );
         # Choose
         my $quote_col = $tc->choose(
             [ @pre, @choices ],
-            { %{$sf->{i}{lyt_h}} }
+            { %{$sf->{i}{lyt_h}}, info => $info }
         );
         if ( ! defined $quote_col ) {
             if ( @bu_col ) {
@@ -328,11 +328,11 @@ sub where {
             next COL;
         }
         if ( $count > 0 && $sql->{where_stmt} !~ /\(\z/ ) { #
-            $ax->print_sql( $sql );
+            my $info = $ax->get_sql_info( $sql );
             # Choose
             $AND_OR = $tc->choose(
                 [ undef, $sf->{and}, $sf->{or} ],
-                { %{$sf->{i}{lyt_h}} }
+                { %{$sf->{i}{lyt_h}}, info => $info }
             );
             if ( ! defined $AND_OR ) {
                 next COL;
@@ -387,12 +387,12 @@ sub group_by {
 
     GROUP_BY: while ( 1 ) {
         $sql->{group_by_stmt} = "GROUP BY " . join ', ', @{$sql->{group_by_cols}};
-        $ax->print_sql( $sql );
+        my $info = $ax->get_sql_info( $sql );
         # Choose
         my @idx = $tc->choose(
             $menu,
-            { %{$sf->{i}{lyt_h}}, meta_items => [ 0 .. $#pre - 1 ], no_spacebar => [ $#pre ], include_highlighted => 2,
-              index => 1 }
+            { %{$sf->{i}{lyt_h}}, info => $info, meta_items => [ 0 .. $#pre - 1 ], no_spacebar => [ $#pre ],
+              include_highlighted => 2, index => 1 }
         );
         if ( ! $idx[0] ) {
             if ( @{$sql->{group_by_cols}} ) {
@@ -448,11 +448,11 @@ sub having {
         if ( $sf->{o}{enable}{parentheses} ) {
             unshift @choices, $unclosed ? ')' : '(';
         }
-        $ax->print_sql( $sql );
+        my $info = $ax->get_sql_info( $sql );
         # Choose
         my $aggr = $tc->choose(
             [ @pre, @choices ],
-            { %{$sf->{i}{lyt_h}} }
+            { %{$sf->{i}{lyt_h}}, info => $info }
         );
         if ( ! defined $aggr ) {
             if ( @bu_col ) {
@@ -478,11 +478,11 @@ sub having {
             next COL;
         }
         if ( $count > 0 && $sql->{having_stmt} !~ /\(\z/ ) {
-            $ax->print_sql( $sql );
+            my $info = $ax->get_sql_info( $sql );
             # Choose
             $AND_OR = $tc->choose(
                 [ undef, $sf->{and}, $sf->{or} ],
-                { %{$sf->{i}{lyt_h}} }
+                { %{$sf->{i}{lyt_h}}, info => $info }
             );
             if ( ! defined $AND_OR ) {
                 next COL;
@@ -547,11 +547,11 @@ sub order_by {
     my @bu;
 
     ORDER_BY: while ( 1 ) {
-        $ax->print_sql( $sql );
+        my $info = $ax->get_sql_info( $sql );
         # Choose
         my $col = $tc->choose(
             [ @pre, @cols ],
-            { %{$sf->{i}{lyt_h}} }
+            { %{$sf->{i}{lyt_h}}, info => $info }
         );
         if ( ! defined $col ) {
             if ( @bu ) {
@@ -579,11 +579,11 @@ sub order_by {
         }
         push @bu, [ $sql->{order_by_stmt}, $col_sep ];
         $sql->{order_by_stmt} .= $col_sep . $col;
-        $ax->print_sql( $sql );
+        $info = $ax->get_sql_info( $sql );
         # Choose
         my $direction = $tc->choose(
             [ undef, $sf->{asc}, $sf->{desc} ],
-            { %{$sf->{i}{lyt_h}} }
+            { %{$sf->{i}{lyt_h}}, info => $info }
         );
         if ( ! defined $direction ){
             ( $sql->{order_by_stmt}, $col_sep ) = @{pop @bu};
@@ -607,11 +607,11 @@ sub limit_offset {
 
     LIMIT: while ( 1 ) {
         my ( $limit, $offset ) = ( 'LIMIT', 'OFFSET' );
-        $ax->print_sql( $sql );
+        my $info = $ax->get_sql_info( $sql );
         # Choose
         my $choice = $tc->choose(
             [ @pre, $limit, $offset ],
-            { %{$sf->{i}{lyt_h}} }
+            { %{$sf->{i}{lyt_h}}, info => $info }
         );
         if ( ! defined $choice ) {
             if ( @bu ) {
@@ -628,10 +628,10 @@ sub limit_offset {
         if ( $choice eq $limit ) {
             #$sql->{limit_stmt} = " LIMIT";
             $sql->{limit_stmt} = "LIMIT";
-            $ax->print_sql( $sql );
+            my $info = $ax->get_sql_info( $sql );
             # Choose_a_number
             my $limit = $tu->choose_a_number( $digits,
-                { cs_label => 'LIMIT: ' }
+                { info => $info, cs_label => 'LIMIT: ' }
             );
             if ( ! defined $limit ) {
                 ( $sql->{limit_stmt}, $sql->{offset_stmt} ) = @{pop @bu};
@@ -646,10 +646,10 @@ sub limit_offset {
                 $sql->{limit_stmt} = "LIMIT " . ( $sf->{o}{G}{max_rows} || '18446744073709551615' ) if $sf->{i}{driver} eq 'mysql';    # 2 ** 64 - 1
             }
             $sql->{offset_stmt} = "OFFSET";
-            $ax->print_sql( $sql );
+            my $info = $ax->get_sql_info( $sql );
             # Choose_a_number
             my $offset = $tu->choose_a_number( $digits,
-                { cs_label => 'OFFSET: ' }
+                { info => $info, cs_label => 'OFFSET: ' }
             );
             if ( ! defined $offset ) {
                 ( $sql->{limit_stmt}, $sql->{offset_stmt} ) = @{pop @bu};
